/*
 * Copyright 2002 Sun Microsystems, Inc. All Rights Reserved.
 */

package javax.mail;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

/**
 * Base class with security related methods that work on JDK 1.1.
 */
class SecuritySupport {

	/*
	 * Make this of type Object so that the verifier won't try to
	 * prove its type, thus possibly trying to load the SecuritySupport12
	 * class.
	 */
	private static final Object securitySupport;

	static {
		SecuritySupport ss = null;
		try {
			Class c = Class.forName("java.security.AccessController");
			// if that worked, we're on 1.2.
/*
			// don't reference the class explicitly so it doesn't
			// get dragged in accidentally.
			c = Class.forName("javax.mail.SecuritySupport12");
			Constructor cons = c.getConstructor(new Class[] { });
			ss = (SecuritySupport)cons.newInstance(new Object[] { });
*/
			/*
			 * Unfortunately, we can't load the class using reflection
			 * because the class is package private.  And the class has
			 * to be package private so the APIs aren't exposed to other
			 * code that could use them to circumvent security.  Thus,
			 * we accept the risk that the direct reference might fail
			 * on some JDK 1.1 JVMs, even though we would never execute
			 * this code in such a case.  Sigh...
			 */
			ss = new SecuritySupport12();
		} catch (Exception ex) {
			// 
		} finally {
			if (ss == null)
				ss = new SecuritySupport();
			securitySupport = ss;
		}
	}

	/**
	 * Return an appropriate instance of this class, depending on whether
	 * we're on a JDK 1.1 or J2SE 1.2 (or later) system.
	 */
	public static final SecuritySupport getInstance() {
		return (SecuritySupport) securitySupport;
	}

	public ClassLoader getContextClassLoader() {
		return null;
	}

	public InputStream getResourceAsStream(final Class c, final String name) throws IOException {
		return c.getResourceAsStream(name);
	}

	public URL[] getResources(final ClassLoader cl, final String name) {
		return null;
	}

	public URL[] getSystemResources(final String name) {
		return null;
	}

	public InputStream openStream(final URL url) throws IOException {
		return url.openStream();
	}

}
