/*
 * Copyright 1997-2002 Sun Microsystems, Inc. All Rights Reserved.
 */

package javax.mail.internet;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * ̃NX MIME p[^ (ƒl̑) ێ܂B
 */
public final class ParameterList {

	private Hashtable list = new Hashtable(); // internal hashtable

	/**
	 * Ȃ̃RXgN^łB
	 */
	public ParameterList() {}

	/**
	 * p[^XggpRXgN^łB
	 * String ̍\͂ƃp[^̎Wyъi[͓Iɍs܂B
	 * \͂Ɏs ParseException X[܂B
	 * ̃p[^Xg͗LłA ParameterList ɍ\͂鎖ɒӂĉB
	 * 
	 * @param s p[^Xg
	 * @throws ParseException \͂Ɏsꍇ
	 */
	public ParameterList(final String s) throws ParseException {
		HeaderTokenizer h = new HeaderTokenizer(s, HeaderTokenizer.MIME);

		do {
			HeaderTokenizer.Token tk = h.next();
			int type = tk.getType();

			if (type == HeaderTokenizer.Token.EOF) // done
				return;

			if ((char)type == ';') {
				// expect parameter name
				HeaderTokenizer.Token tk2 = h.next();
				// tolerate trailing semicolon, even though it violates the spec
				if (tk2.getType() == HeaderTokenizer.Token.EOF)
					return;
				// parameter name must be a MIME Atom
				if (tk2.getType() != HeaderTokenizer.Token.ATOM)
					throw new ParseException();
				String name = tk2.getValue().toLowerCase();

				// expect '='
				tk2 = h.next();
				if ((char)tk2.getType() != '=')
					throw new ParseException();

				// expect parameter value
				tk2 = h.next();
				int type2 = tk2.getType();
				// parameter value must be a MIME Atom or Quoted String
				if (type2 != HeaderTokenizer.Token.ATOM && type2 != HeaderTokenizer.Token.QUOTEDSTRING)
					throw new ParseException();

				list.put(name, tk2.getValue());
			} else
				throw new ParseException();
		} while (true);
	}

	/**
	 * ̃Xgɂp[^̐Ԃ܂B
	 * 
	 * @return p[^̐
	 */
	public int size() {
		return list.size();
	}

	/**
	 * w肳ꂽp[^̒lԂ܂B
	 * p[^͑啶ƏʂȂɒӂĉB
	 * 
	 * @param name p[^
	 * @return p[^̒lBp[^݂Ȃꍇ <code>null</code> Ԃ܂B 
	 */
	public String get(final String name) {
		return (String) list.get(name.trim().toLowerCase());
	}

	/**
	 * p[^ݒ肵܂B̃p[^ɑ݂ꍇA̐VlŒu܂B
	 * 
	 * @param name p[^̖O
	 * @param value p[^̒l
	 */
	public void set(final String name, final String value) {
		list.put(name.trim().toLowerCase(), value);
	}

	/**
	 * w肳ꂽp[^ ParameterList 폜܂B
	 * p[^݂ȂꍇÃ\bh͉s܂B
	 * 
	 * @param name p[^̖O
	 */
	public void remove(final String name) {
		list.remove(name.trim().toLowerCase());
	}

	/**
	 * ̃Xg̑Sp[^̖O̗񋓂Ԃ܂B
	 * 
	 * @return ̃Xg̑Sp[^̗
	 */
	public Enumeration getNames() {
		return list.keys();
	}

	/**
	 *  ParameterList  MIME String ɕϊ܂B
	 * ꂪ̃Xg̏ꍇA̕񂪕Ԃ܂B
	 * 
	 * @return String
	 */
	public String toString() {
		return toString(0);
	}

	/**
	 *  ParameterList  MIME String ɕϊ܂B
	 * ꂪ̃Xg̏ꍇA̕񂪕Ԃ܂B
	 * 
	 * 'used' p[^́Aʂ̃p[^Xg}tB[hɂāA
	 * ɐLĂ镶ʒu̐w肵܂B
	 * ʂ̃p[^Xg}ꏊʂׂɎgp܂B
	 * 
	 * @param used p[^Xg}tB[hŊɎgpĂ镶ʒu̐
	 * @return String
	 */  
	public String toString(int used) {
		StringBuffer sb = new StringBuffer();
		Enumeration e = list.keys();

		while (e.hasMoreElements()) {
			String name = (String) e.nextElement();
			String value = quote((String)list.get(name));
			sb.append("; ");
			used += 2;
			int len = name.length() + value.length() + 1;
			if (used + len > 76) { // overflows ...
				sb.append("\r\n\t"); // .. start new continuation line
				used = 8; // account for the starting <tab> char
			}
			sb.append(name).append('=');
			used += name.length() + 1;
			if (used + value.length() > 76) { // still overflows ...
				// have to fold value
				String s = MimeUtility.fold(used, value);
				sb.append(s);
				int lastlf = s.lastIndexOf('\n');
				if (lastlf >= 0)	// always true
					used += s.length() - lastlf - 1;
				else
					used += s.length();
			} else {
				sb.append(value);
				used += value.length();
			}
		}
		return sb.toString();
	}

	// Quote a parameter value token if required.
	private String quote(final String value) {
		return MimeUtility.quote(value, HeaderTokenizer.MIME);
	}

}
